<?php
class CacheTest extends PHPUnit_Framework_TestCase
{
	function setUp()
	{
		evil_global_setup();
		set_user('host_7', array('op5_admins'));
		global $HOME, $IMPORTING;
		$cfgs = array($HOME.'mangleconfig.cfg', $HOME.'testconfig.cfg');
		require_once('include/import.inc.php');
		$this->assertTrue(
			import_config('phptap', $cfgs),
			"import_config('phptap', '" . print_r($cfgs, true) . "')"
		);
		$IMPORTING = false;
		$this->assertNotSame(false, file_put_contents('custom_config.php', '<?php $DEBUG = TRUE; $HOME = "'.realpath(getenv('WORK_DIR')).'/"; $Config["main"] = $HOME."nagios.cfg"; $Config["cgi"] = $HOME."cgi.cfg";'));
	}

	function tearDown()
	{
		$this->assertTrue(unlink('custom_config.php'));
		nacoma_db::exec_query('DELETE FROM changelog');
		evil_global_teardown();
	}

	function testIfCacheWhenRelationChanges()
	{
		nacoma_db::exec_query('TRUNCATE TABLE changelog');
		$changes = new ChangeLog_nacoma_Model();
		$this->assertEquals($changes->is_recache_required(), false, 'Recache is not required when no changes');
		$host = new host_nacoma_Model(32);
		$this->assertTrue(is_numeric($host->id), 'Host exists');
		$contact = new contact_nacoma_Model(5);
		$this->assertTrue(is_numeric($contact->id), 'Contact exists');
		$host->obj['contacts'][] = $contact->id;
		$host->save_object();
		$this->assertEmpty($host->validation_error_list, 'User can save');
		$changes = new ChangeLog_nacoma_Model();
		$this->assertEquals($changes->is_recache_required(), true, 'Recache is required when host has new contact');

		revert_users_changes();
	}

	function testIfCacheWhenNewObject()
	{
		nacoma_db::exec_query('TRUNCATE TABLE changelog');
		$cg = new contactgroup_nacoma_Model();
		$cg->obj['contactgroup_name'] = 'contactgroup_name';
		$cg->obj['alias'] = 'contactgroup_name';
		$cg->obj['members'] = array(1);
		$cg->save_object();
		$this->assertEmpty($cg->validation_error_list, 'User should be able to save, got: '.var_export($cg->validation_error_list, true));
		$changes = new ChangeLog_nacoma_Model();
		$this->assertEquals($changes->is_recache_required(), true, 'Recache is required when new contactgroup');
		revert_users_changes();
	}

	function testIfCacheWhenUnrelatedChanges()
	{
		nacoma_db::exec_query('TRUNCATE TABLE changelog');
		$c = new command_nacoma_Model();
		$c->obj['command_name'] = 'command_name';
		$c->obj['command_line'] = 'a';
		$c->save_object();
		$this->assertEmpty($c->validation_error_list, 'User should be able to save: '.var_export($c->validation_error_list, true));

		$hg = new hostgroup_nacoma_Model(4);
		$hg->obj['members'][] = 3;
		$hg->save_object();
		$this->assertEmpty($hg->validation_error_list, 'User can save');

		$changes = new ChangeLog_nacoma_Model();
		$this->assertEquals($changes->is_recache_required(), false, 'Recache is not required when unrelated changes');
		revert_users_changes();
	}
}