<?php
/**
 * Tests to make sure an object's related objects and relations to other
 * objects are deleted with the object itself.
 * This is relevant to test, as errors here have been known to make nacoma
 * completely unable to export it's settings.
 */
class DeleteTest extends PHPUnit_Framework_TestCase {

	protected function setUp() {
		evil_global_setup();
		set_user('host_7', array('op5_admins'));
		require_once('include/import.inc.php');
		global $HOME, $IMPORTING;
		$cfgs = array($HOME.'mangleconfig.cfg', $HOME.'testconfig.cfg');
		$this->assertTrue(
			import_config('phptap', $cfgs),
			"import_config('phptap', '" . print_r($cfgs, true) . "')"
		);
		$IMPORTING = false;

		$this->host = new host_nacoma_Model();
		$this->host->get_default_object();
		$this->host->obj['host_name'] = 'dead object';
		$this->host->obj['alias'] = 'dead object';
		$this->host->obj['address'] = 'address';
		$this->service = new service_nacoma_Model();
		$this->service->get_default_object();
		$this->service->obj['service_description'] = 'dead';
		$this->service->obj['check_command'] = 1;
	}

	protected function tearDown() {
		nacoma_db::exec_query('DELETE FROM management_pack WHERE management_pack_name = "an mgmt pack"');
		evil_global_teardown();
	}

	private function relationTest($obj, $rels) {
		$obj_id = $obj->id;
		foreach ($rels as $rel => $cnt) {
			$res = nacoma_db::exec_query('SELECT * FROM '.get_junction_table_name($obj->type, $rel)." WHERE $obj->type=$obj_id");
			$ids = $res->fetchAll(PDO::FETCH_ASSOC);
			$this->assertCount($cnt, $ids, "Unexpected number of $rel relations for {$obj->type} \"{$obj->get_object_name()}\": got ".var_export($ids, true));
		}
	}

	private function saveAndTest($obj, $rels=array()) {
		$this->assertTrue($obj->save_object(), 'Failed to save object: '.var_export($obj->validation_error_list, true));
		$obj_id = $obj->id;
		$res = nacoma_db::exec_query("SELECT 1 FROM $obj->db_table WHERE id=$obj_id");
		$this->assertCount(1, $res->fetchAll(PDO::FETCH_NUM), "Expected to find the $obj->type object");
		$this->relationTest($obj, $rels);
	}

	private function deleteAndTest($obj, $rels=array()) {
		$obj_id = $obj->id;
		$this->assertTrue($obj->banish_object(), "Couldn't delete object: ".$obj->get_deletion_message());
		$res = nacoma_db::exec_query("SELECT 1 FROM $obj->db_table WHERE id=$obj_id");
		$this->assertEquals(count($res->fetchAll(PDO::FETCH_NUM)), 0, "$obj->type object was deleted");
		foreach ($rels as $rel => $cnt)
			$rels[$rel] = 0;
		$this->relationTest($obj, $rels);
	}

	public function testDeleteHostWithRelations() {
		$this->host->obj['parents'] = array(36);
		$this->host->obj['hostgroups'] = array(1);
		$this->host->obj['contacts'] = array(1, 2);
		$this->host->obj['contact_groups'] = array(1);
		$this->saveAndTest($this->host, array('contacts' => 2, 'contact_groups' => 1, 'parents' => 1, 'hostgroup'=> 1));
		$this->deleteAndTest($this->host, array('contacts' => 2, 'contact_groups' => 1, 'parents' => 1, 'hostgroup'=> 1));
	}

	public function testDeleteServiceWithRelations() {
		$member = new host_nacoma_Model('host_9');
		$this->assertTrue(is_numeric($member->id), 'Member host exists');
		$this->service->obj['contacts'] = array(1,2);
		$this->service->obj['contact_groups'] = array(1);
		$this->service->obj['servicegroups'] = array(1);
		$this->service->obj['host_name'] = $member->id;
		$this->saveAndTest($this->service, array('contacts' => 2, 'contact_groups' => 1, 'servicegroup' => 1));
		$this->deleteAndTest($this->service, array('contacts' => 2, 'contact_groups' => 1, 'servicegroup' => 1));
	}

	public function testDeleteHostGroupWithRelations()	{
		$member = new host_nacoma_Model('host_9');
		$this->assertTrue(is_numeric($member->id), 'Member host exists');
		$hg = new hostgroup_nacoma_Model();
		$hg->obj['hostgroup_name'] = 'name';
		$hg->obj['alias'] = 'name';
		$hg->obj['members'] = array($member->id);
		$this->assertTrue($hg->save_object(), 'Hostgroup should save, got '.var_export($hg->validation_error_list, true));
		$this->relationTest($hg, array('members' => 1), 'Only one member should be created for new hostgroup');
		$this->host->obj['hostgroups'] = array($hg->id);
		$this->assertTrue($this->host->save_object(), 'Host should be saved, got '.var_export($this->host->validation_error_list, true));
		$hg = new hostgroup_nacoma_Model($hg->id);
		$this->saveAndTest($hg, array('members' => 2));
		$this->deleteAndTest($hg, array('members' => 2));

		$this->host->banish_object();
	}

	public function testDeleteHostWithService() {
		$this->saveAndTest($this->host);
		$this->service->obj['host_name'] = $this->host->id;
		$this->service->obj['contacts'] = array(1, 2);
		$this->service->obj['contact_groups'] = array(1);
		$this->saveAndTest($this->service, array('contacts'=>2, 'contact_groups'=>1));
		$this->host->banish_object();
		$res = nacoma_db::exec_query("SELECT 1 FROM service WHERE service_description = 'dead'");
		$this->assertEmpty($res->fetchAll(PDO::FETCH_NUM), 'Service is gone');
		$this->relationTest($this->service, array('contact'=>0,'contactgroup'=>0));
	}

	public function testDeleteHostWithServiceEscalation() {
		$this->saveAndTest($this->host);
		$this->service->obj['host_name'] = $this->host->id;
		$this->saveAndTest($this->service);
		$sesc = new serviceescalation_nacoma_Model();
		$sesc->get_default_object();
		$sesc->obj['service'] = $this->service->id;
		$sesc->obj['contacts'] = array(1,2);
		$this->saveAndTest($sesc, array('contacts'=>2));
		$this->host->banish_object();
		$res = nacoma_db::exec_query("SELECT 1 FROM service WHERE id=".$this->service->id);
		$this->assertEmpty($res->fetchAll(PDO::FETCH_NUM), 'Service is gone');
		unset($res);
		$res = nacoma_db::exec_query("SELECT 1 FROM serviceescalation WHERE id=".$sesc->id);
		$this->assertEmpty($res->fetchAll(PDO::FETCH_NUM), 'Serviceescalation is gone');
		$this->relationTest($sesc, array('contacts'=>0));
	}

	public function testDeleteWithReqAltDeps() {
		$hg = new hostgroup_nacoma_Model();
		$hg->get_default_object();
		$hg->obj['hostgroup_name'] = 'test hostgroup';
		$hg->obj['alias'] = 'test hostgroup';
		$hg->obj['members'] = array(43);
		$hg->save_object();
		$this->assertEmpty($hg->validation_error_list, "New hostgroup saved ok");
		$this->service->set_master_object_type('hostgroup');
		$this->service->obj['hostgroup_name'] = $hg->id;
		$this->service->save_object();
		$this->assertEmpty($this->service->validation_error_list, 'New service saved ok');
		$this->assertTrue(
			$hg->banish_object(),
			"Deleting hostgroup should be OK, got" .
				print_r($hg->delete_message, true)
		);

		$res = nacoma_db::exec_query('SELECT COUNT(1) FROM service WHERE id = '.$this->service->id);
		$row = $res->fetch(PDO::FETCH_NUM);
		$this->assertEquals(0, $row[0], "Service was deleted when it's parent object was");
	}

	public function testDeleteHostDeps() {
		$hg = new hostgroup_nacoma_Model();
		$hg->get_default_object();
		$hg->obj['hostgroup_name'] = 'test hostgroup';
		$hg->obj['alias'] = 'test hostgroup';
		$hg->obj['members'] = array(43);
		$hg->save_object();
		$this->assertEmpty($hg->validation_error_list, "New hostgroup saved ok");

		$host = new host_nacoma_Model();
		$host->get_default_object();
		$host->obj['host_name'] = 'test object';
		$host->obj['alias'] = 'test object';
		$host->obj['address'] = 'address';
		$host->save_object();
		$this->assertEmpty($host->validation_error_list, 'New host saved ok');

		$this->host->save_object();
		$this->assertEmpty($this->host->validation_error_list, 'New host saved ok');

		$hd = new hostdependency_nacoma_Model();
		$hd->get_default_object();
		$hd->obj['host_name'] = $host->id;
		$hd->obj['hostgroup_name'] = $hg->id;
		$hd->obj['dependent_host_name'] = $this->host->id;
		$hd->save_object();
		$this->assertEmpty($hd->validation_error_list, 'New hostdependency saved ok');

		$hgobj = $hg->obj;
		$hg->banish_object();
		$nhd = new hostdependency_nacoma_Model($hd->id);
		$this->assertTrue(!empty($nhd->obj), 'Hostdependency still exists');
		$res = nacoma_db::exec_query('SELECT host_name, hostgroup_name FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_ASSOC);
		$this->assertEmpty($row['hostgroup_name'], 'Hostgroup cleared in dependency when deleted');
		$this->assertEquals($row['host_name'], $host->id, 'Host not cleared in dependency');
		unset($res);

		$hostobj = $host->obj;
		$host->banish_object();
		$nhd = new hostdependency_nacoma_Model($hd->id);
		$this->assertEmpty($nhd->obj, "Hostdependency doesn't still exist");
		$res = nacoma_db::exec_query('SELECT COUNT(1) FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_NUM);
		$this->assertEquals($row[0], 0, "Hostgroup deleted when both it's req_alts are");
		unset($res);

		$hg->set_object_id('new');
		$hg->obj = $hgobj;
		$hg->save_object();
		$hd->obj['hostgroup_name'] = $hg->id;
		$host->set_object_id('new');
		$host->obj = $hostobj;
		$host->save_object();
		$hd->obj['host_name'] = $host->id;
		$hd->save_object();
		$this->assertEmpty($hd->validation_error_list, 'Hostdependency updated ok');

		$hostobj = $host->obj;
		$host->banish_object();
		$nhd = new hostdependency_nacoma_Model($hd->id);
		$this->assertTrue(!empty($nhd->obj), 'Hostdependency still exists');
		$res = nacoma_db::exec_query('SELECT host_name, hostgroup_name FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_ASSOC);
		$this->assertEmpty($row['host_name'], 'Host cleared in dependency when deleted');
		$this->assertEquals($row['hostgroup_name'], $hg->id, 'Hostgroup not cleared in dependency');
		unset($res);

		$hg->banish_object();
		$nhd = new hostdependency_nacoma_Model($hd->id);
		$this->assertEmpty($nhd->obj, "Hostdependency doesn't still exist");
		$res = nacoma_db::exec_query('SELECT COUNT(1) FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_NUM);
		$this->assertEquals($row[0], 0, "Hostdependency deleted when both it's req_alts are");
		unset($res);
		$this->host->banish_object();
	}

	public function testDeleteReasonText() {
		$hg = new hostgroup_nacoma_Model();
		$hg->get_default_object();
		$hg->obj['hostgroup_name'] = 'test hostgroup';
		$hg->obj['alias'] = 'test hostgroup';
		$hg->save_object();
		$this->assertEmpty($hg->validation_error_list, "New hostgroup saved ok");

		$host = new host_nacoma_Model();
		$host->obj = $this->host->obj;
		$host->obj['hostgroups'] = array($hg->id);
		$host->save_object();
		$this->assertEmpty($host->validation_error_list, 'New host saved ok');

		$this->assertSame($host->delete_object(), false, "Deleting host with relation shouldn't work without warnings");
		$this->assertSame(preg_match('/(affect the following objects).+(hostgroup test hostgroup)/', $host->delete_message), 1, "Expected 'test hostgroup' to be mentioned as a related object, got: $host->delete_message");
		$this->assertSame(preg_match('/(completely remove the following objects)/', $host->delete_message), 0, "Expected no slave objects, got: $host->delete_message");

		$service = new service_nacoma_Model();
		$service->obj = $this->service->obj;
		$service->obj['host_name'] = $host->id;
		$service->save_object();
		$this->assertEmpty($service->validation_error_list, 'New host service ok');

		$this->assertSame($host->delete_object(), false, "Deleting host with slave shouldn't work without warnings");
		$this->assertSame(preg_match('/(completely remove the following objects).+(service dead object;dead).+(affect the following objects).+(hostgroup test hostgroup)/', $host->delete_message), 1, "Expected 'test hostgroup' to be mentioned as a related object and 'dead' slave to be mentioned as a slave, got: $host->delete_message");

		$hg->banish_object();
		$host->banish_object();
	}

	public function testDeleteMgmtPack() {
		$hg = new hostgroup_nacoma_Model();
		$hg->get_default_object();
		$hg->obj['hostgroup_name'] = 'test hostgroup';
		$hg->obj['alias'] = 'test hostgroup';
		$hg->save_object();
		$this->assertEmpty($hg->validation_error_list, "New hostgroup saved ok");

		$mgmt = new management_pack_nacoma_Model();
		$mgmt->get_default_object();
		$mgmt->obj['management_pack_name'] = 'an mgmt pack';
		$mgmt->obj['hostgroup'] = $hg->obj['hostgroup_name'];
		$p = $mgmt->get_properties();
		$mgmt->obj['icon'] = current($p['special']['icon']);
		$mgmt->obj['description'] = 'the pack has a purpose';
		$this->assertTrue($mgmt->save_object(), 'Saving new management pack should be ok, got '.var_export($mgmt->validation_error_list, true));
		$this->assertEmpty($mgmt->validation_error_list, "There should not be errors saving new management pack");

		$this->assertSame($hg->delete_object(), false, "Deleting a the hostgroup with a mgmt pack should fail");
		$this->assertSame(preg_match('/(You may not delete this hostgroup).+(an mgmt pack)/', $hg->delete_message), 1, "When deleting hostgroup with mgmtpack, the mgmtpack should block deletion: ".$hg->delete_message);

		$this->assertSame($hg->banish_object(), false, "Banishing a hostgroup with a mgmt pack should fail");
		$this->assertSame($mgmt->banish_object(), true, "Banishing a mgmt pack should work");
		$this->assertSame($hg->banish_object(), true, "Banishing a hostgroup without a mgmt pack should work");
	}

	public function testDeleteGraphCollection() {
		$gc = new graph_collection_nacoma_Model();
		$gc->obj['page_name'] = 'graph1';
		$gc->obj['services'] = array('host1', 'host1;Mem Usage', 'host2', 'host2;CPU - Idle');
		$gc->obj['use_regex'] = false;

		$this->assertFalse($gc->get_object(), 'Got a graph collection object before saving ');
		$this->assertTrue($gc->save_object(), 'Saving new graph collection should be ok, got '.var_export($gc->validation_error_list, true));
		$this->assertSame($gc->get_object(), $gc->obj, 'Both graph collection objects should be same');

		$this->assertTrue($gc->banish_object(), 'Could not banish graph collection object. '.var_export($gc->validation_error_list, true));
		$this->assertFalse($gc->get_object(), 'Got a graph collection object after banishing ');
	}

	public function testDeleteCombinedGraph() {
		$cg = new combined_graph_nacoma_Model();
		$cg->obj['title'] = 'graph1';
		$cg->obj['comment'] = 'comment';
		$cg->obj['type'] = 'area';
		$cg->obj['services'] = array('host1', 'host1;Mem Usage', 'host2', 'host2;CPU - Idle');
		$this->assertFalse($cg->get_object(), 'Got a combined graph object before saving');
		$this->assertTrue($cg->save_object(), 'Saving a combined graph should be ok, got '.var_export($cg->validation_error_list, true));
		$this->assertSame($cg->get_object(), $cg->obj, 'Both combined graph objects should be same');
		$this->assertTrue($cg->banish_object(), 'Could not banish combined graph object. '.var_export($cg->validation_error_list, true));
		$this->assertFalse($cg->get_object(), 'Got a combined graph object after banishing');
	}

	/**
	 * "Normal" service templates should be deletable.
	 */
	public function testDeleteServiceTemplate() {
		$st = new service_template_nacoma_Model();
		$st->obj['name'] = 'test';
		$this->assertTrue($st->save_object(), 'Failed to save service template.');
		$this->assertTrue($st->delete_object(), 'Failed to delete service template.');
	}

	/**
	 * Default templates should not be deletable.
	 */
	public function testDeleteDefaultTemplate() {
		// "default-service" is only one example of a default template.
		$st = new service_template_nacoma_Model('default-service');
		$this->assertFalse(
			$st->delete_object(),
			'The template "default-service" should not be deletable.'
		);
	}

	/**
	 * Provides "protected" BSM objects for the test below.
	 */
	public function protectBsmObjects() {
		$st = new service_template_nacoma_Model();
		$st->obj['name'] = 'bizproc_service';

		$c = new command_nacoma_Model();
		$c->obj['command_name'] = 'bizproc_pas';
		$c->obj['command_line'] = 'echo "Service not intended for active checks"';

		return array(
			array($st),
			array($c)
		);
	}

	/**
	 * Some Nacoma objects should not be deletable since they are used by BSM.
	 * @dataProvider protectBsmObjects
	 */
	public function testDeleteProtectedBsmObjects($obj) {
		$save_ok = $obj->save_object();

		// Convert "saving array" to a readable string so that we get a somewhat
		// good error message if save_object() fails.
		$error_msg = '';
		foreach ($obj->validation_error_list as $prop => $desc) {
			$error_msg .= '"' . $prop . '": ' . $desc . "\n";
		}

		$this->assertTrue($save_ok, 'Failed to save command. ' . $error_msg);
		$this->assertFalse(
			$obj->delete_object(),
			'Th Nacoma object should not be deletable.'
		);
	}

}
