<?php

class HookTest extends PHPUnit_Framework_TestCase
{
	function setUp()
	{
		evil_global_setup();
		set_user('host_7', array('op5_admins'));
		require_once('include/import.inc.php');
		global $HOME, $IMPORTING;
		$cfgs = array($HOME.'testconfig.cfg');
		$this->assertTrue(
			import_config('phptap', $cfgs),
			"import_config('phptap', '" . print_r($cfgs, true) . "')"
		);
		$IMPORTING = false;

		global $HOOKDIR;
		$HOOKDIR = '/tmp/nacoma-hook-test/';
		$this->assertTrue(mkdir($HOOKDIR.'save/', 0777, true));
		$this->assertTrue(mkdir($HOOKDIR.'save_denormalized/', 0777, true));
	}

	function tearDown()
	{
		// if anything failed, this will lock up for 5 minutes:
		nacoma_db::exec_query('TRUNCATE TABLE objlocks');

		global $HOOKDIR;
		# do some minor sanity checking on $HOOKDIR, just in case - it is a
		# global, after all... (and changing it is a bug)
		$this->assertTrue(is_dir($HOOKDIR));
		$this->assertTrue(substr($HOOKDIR, 0, 4) == '/tmp');

		foreach (scandir($HOOKDIR) as $hooktype) {
			if (in_array($hooktype, array('.', '..')))
				continue;
			if (!is_dir($HOOKDIR . $hooktype)) {
				$this->assertTrue(unlink($HOOKDIR . $hooktype));
				continue;
			}
			foreach (scandir($HOOKDIR . $hooktype) as $file) {
				if (in_array($file, array('.', '..')))
					continue;
				$this->assertTrue(is_file($HOOKDIR . $hooktype . '/' . $file));
				$this->assertTrue(unlink($HOOKDIR . $hooktype . '/' . $file));
			}
			$this->assertTrue(rmdir($HOOKDIR . $hooktype));
		}
		$this->assertTrue(rmdir($HOOKDIR));
		evil_global_teardown();
	}

	function testSaveMy()
	{
		global $HOOKDIR;
		$fd = fopen($HOOKDIR.'save/testhook', 'w');
		fwrite($fd, "#!/bin/sh\nwhile read line; do echo \$line > $HOOKDIR/testhook_out; done");
		fclose($fd);
		chmod($HOOKDIR.'save/testhook', 777);
		$hst = new host_nacoma_Model('host_10');
		$this->assertGreaterThan(0, $hst->id);
		$hst->obj['host_name'] = 'a new hostname';
		$this->assertTrue($hst->save_object());
		$changes = new ChangeLog_nacoma_Model();
		$this->assertSame(true, do_export($changes));

		$this->assertFileExists("$HOOKDIR/testhook_out");
		$hookres = file_get_contents("$HOOKDIR/testhook_out");
		$this->assertContains("object_type='host'", $hookres);
		$this->assertContains("oldname='host_10'", $hookres);
		$this->assertContains("newname='a new hostname'", $hookres);
		$this->assertContains("username='host_7'", $hookres);
		$this->assertContains("pre='host_name=host_10;'", $hookres);
		$this->assertContains("post='host_name=a new hostname;'", $hookres);
	}

	function testSaveAll()
	{
		global $HOOKDIR;
		$fd = fopen($HOOKDIR.'save/testhook', 'w');
		fwrite($fd, "#!/bin/sh\nwhile read line; do echo \$line > $HOOKDIR/testhook_out; done");
		fclose($fd);
		chmod($HOOKDIR.'save/testhook', 777);
		$hst = new host_nacoma_Model('host_11');
		$this->assertGreaterThan(0, $hst->id);
		$hst->obj['host_name'] = 'another new hostname';
		$this->assertTrue($hst->save_object());
		$changes = new ChangeLog_nacoma_Model();
		$this->assertSame(true, do_export(true));

		$this->assertFileExists("$HOOKDIR/testhook_out");
		$hookres = file_get_contents("$HOOKDIR/testhook_out");
		$this->assertContains("object_type='host'", $hookres);
		$this->assertContains("oldname='host_11'", $hookres);
		$this->assertContains("newname='another new hostname'", $hookres);
		$this->assertContains("username='host_7'", $hookres);
		$this->assertContains("pre='host_name=host_11;'", $hookres);
		$this->assertContains("post='host_name=another new hostname;'", $hookres);
	}

	function testSaveDenormalized()
	{
		global $HOOKDIR;
		$fd = fopen($HOOKDIR.'save_denormalized/testhook', 'w');
		fwrite($fd, "#!/bin/sh\nwhile read line; do echo \$line > $HOOKDIR/testhook_out; done");
		fclose($fd);
		chmod($HOOKDIR.'save_denormalized/testhook', 777);
		$hst = new host_nacoma_Model('host_12');
		$this->assertGreaterThan(0, $hst->id);
		$hst->obj['host_name'] = 'even more hostnames';
		$this->assertTrue($hst->save_object(), "Saving host should work, got ".var_export($hst->validation_error_list, true));
		$changes = new ChangeLog_nacoma_Model();
		$this->assertSame(true, do_export($changes));

		$this->assertFileExists("$HOOKDIR/testhook_out");
		$hookres = file_get_contents("$HOOKDIR/testhook_out");
		$this->assertContains("object_type='host'", $hookres);
		$this->assertContains("oldname='host_12'", $hookres);
		$this->assertContains("newname='even more hostnames'", $hookres);
		$this->assertContains("username='host_7'", $hookres);
		$this->assertContains("pre='host_name=host_12;'", $hookres);
		$this->assertContains("post='host_name=even more hostnames;'", $hookres);
	}

	function testMuchOutputStdoutStderr()
	{
		require_once __DIR__."/../../include/hooks.inc.php";
		global $HOOKDIR;
		$hook_name = $HOOKDIR.'save/spammy_hook';
		// Since Linux 2.6.11
		$max_pipe_size = 64 * 1024;
		// Since Linux 2.6.35:
		if(file_exists("/proc/sys/fs/pipe-max-size")) {
			$max_pipe_size = (int) trim(file_get_contents("/proc/sys/fs/pipe-max-size"));
		}
		$exceeding = $max_pipe_size + 1;
		$hook_contents = <<<EOF
#!/usr/bin/env php
<?php

\$stdout = fopen('php://stdout', 'w');
\$stderr = fopen('php://stderr', 'w');

\$pipe_buffer_size_plus_some_more = $exceeding;

fwrite(\$stdout, str_repeat('a', \$pipe_buffer_size_plus_some_more)); fwrite(\$stderr, str_repeat('a', \$pipe_buffer_size_plus_some_more));

EOF;
		$this->assertTrue(false !== file_put_contents($hook_name, $hook_contents));
		$this->assertTrue(chmod($hook_name, 777));

		$changes = array(
			array(
				'id' => '2',
				'object_type' => 'host',
				'object_id' => '2',
				'oldname' => 'asdf nytt namn',
				'newname' => 'asdf',
				'oldstate' => 'a:3:{s:9:"host_name";s:14:"asdf nytt namn";s:8:"template";i:1;s:7:"file_id";i:3;}',
				'newstate' => 'a:3:{s:8:"template";s:1:"1";s:9:"host_name";s:4:"asdf";s:7:"file_id";s:1:"3";}',
				'username' => 'admin1',
				'time' => '2015-10-12 07:18:40',
			)
		);
		$this->expectOutputString(str_repeat("a", $exceeding * 2));

		// previously, this step would hang, see MON-7440
		$this->assertTrue(run_hooks('save', $changes));
	}
}
