<?php
// because there's some stuff that just too obscure to create separate test files for

class MiscTest extends PHPUnit_Framework_TestCase
{
	protected $backupGlobalsBlacklist = array('master_ary', 'Supported_Object_Types');
	function setUp()
	{
		evil_global_setup();
		set_user('host_7', array('op5_admins'));
		global $HOME, $IMPORTING;
		$cfgs = array($HOME.'mangleconfig.cfg', $HOME.'testconfig.cfg');
		require_once('include/import.inc.php');
		$this->assertTrue(
			import_config('phptap', $cfgs),
			"import_config('phptap', '" . print_r($cfgs, true) . "')"
		);
		$IMPORTING = false;
		$this->assertNotSame(false, file_put_contents('custom_config.php', '<?php $DEBUG = TRUE; $HOME = "'.realpath(getenv('WORK_DIR')).'/"; $Config["main"] = $HOME."nagios.cfg"; $Config["cgi"] = $HOME."cgi.cfg";'));
	}

	function tearDown()
	{
		$this->assertTrue(unlink('custom_config.php'));
		nacoma_db::exec_query('DELETE FROM changelog');
		evil_global_teardown();
	}

	function testGetMerlinNodes() {
		$test_node_show_data = <<<EOS

# poller_name1
ADDRESS=poller_address1
TYPE=poller
PORT=15551
HOSTGROUP=hostgroup_1
NAME=poller_name1

# poller_name2
ADDRESS=poller_address2
TYPE=poller
PORT=15551
HOSTGROUP=hostgroup_4
NAME=poller_name2

# peer_name
ADDRESS=peer_address
TYPE=peer
PORT=15551
NAME=peer_name
EOS
		;
		set_user('host_7', array('op5_admins'));
		naemon_nacoma_Model::$objects_for_user = array();
		$output = parse_merlin_nodes($test_node_show_data);
		$this->assertEquals($output, array('This server', 'poller_name1', 'poller_name2', 'peer_name'), 'get_merlin_nodes should return all nodes to super-users');

		set_user('contact_limited', array('contact_limited'));
		naemon_nacoma_Model::$objects_for_user = array();
		$output = parse_merlin_nodes($test_node_show_data);
		$this->assertEquals($output, array('This server', 'poller_name2', 'peer_name'), "get_merlin_nodes shouldn't return all nodes to limited users, but it should return some");
	}

	function invalidProvider()
	{
		return array(
			array(null),
			array(array('management_pack' => array('management_pack' => array('management_pack_name' => 'foo')))),
			array(array('management_pack' => array('commands' => array()))),
			array('this is not a management pack'),
		);
	}

	/**
 	 * @dataProvider invalidProvider
	 * @expectedException ManagementPackManagementValidationException
	 */
	function testManagementPackManagementValidation($pack)
	{
		management_pack_management_nacoma_Model::validate_input($pack);
	}


	function trixy_check_helper($pack, $descr)
	{
		$mpm = new management_pack_management_nacoma_Model();
		$res = $mpm->is_imported($pack);
		$this->assertTrue(
			is_array($res),
			"Expected management pack $descr to be trixsy, got " .
				print_r($res, true)
		);
		$res = $mpm->import($pack);
		$this->assertSame($res, false, "Importing the management pack $descr force-free should complain, got ".
			var_export(is_object($res)?$res->validation_error_list:$res, true));
		$res = $mpm->import($pack, true);
		$this->assertTrue($res > 0, "Importing the management pack $descr with force should work, got ".
			var_export(is_object($res)?$res->validation_error_list:$res, true));
		$this->assertSame($mpm->is_imported($pack), 1, "The management pack $descr should be considered imported when asking again");
	}
	function unimported_check_helper($pack, $descr)
	{
		$mpm = new management_pack_management_nacoma_Model();
		$this->assertSame($mpm->is_imported($pack), 0, "The management pack $descr should not be imported");
		$res = $mpm->import($pack);
		$this->assertTrue($res > 0, "Importing the management pack $descr should work, got ".
			var_export(is_object($res)?$res->validation_error_list:$res, true));
		$this->assertSame($mpm->is_imported($pack), 1, "The management pack $descr should be considered imported when asking again");
	}
	function cleanup_helper($pack)
	{
		set_user('host_7', array('op5_admins'));
		foreach ($pack['commands'] as $cmd) {
			$command = new command_nacoma_Model($cmd['command_name']);
			$this->assertTrue($command->banish_object(), 'Deleting command should work, got '.
				$command->get_deletion_message());
		}
		foreach ($pack['services'] as $svc) {
			$service = new service_nacoma_Model($svc['hostgroup_name'].';'.$svc['service_description']);
			$this->assertTrue($service->banish_object(), 'Deleting service should work, got '.
				$service->get_deletion_message());
		}
		$mgmt = new management_pack_nacoma_Model($pack['management_pack']['management_pack_name']);
		$this->assertTrue($mgmt->banish_object(), 'Deleting management pack should work, got '.
			$mgmt->get_deletion_message());
		$hg = new hostgroup_nacoma_Model($pack['hostgroup']['hostgroup_name']);
		$this->assertTrue($hg->banish_object(), 'Deleting hostgroup should work, got '.
			$hg->get_deletion_message());
	}

	function testManagementPackIsImported()
	{
		set_user('host_7', array('op5_admins'));
		$real_pack = array(
			'management_pack' => array(
				'management_pack_name' => 'TEST_PACK',
				'description' => "This is not the pack you're looking for",
				'icon' => 'server.png',
				'hostgroup' => 'TEST_HG',
				'_VAR' => 'value',
			),
			'hostgroup' => array(
				'hostgroup_name' => 'TEST_HG',
				'notes' => 'what?',
			),
			'services' => array(
				array(
					'service_description' => 'TEST_SVC1',
					'hostgroup_name' => 'TEST_HG',
					'check_command' => 'TEST_COMMAND1',
					'notes' => 'no reason',
				),
				array(
					'service_description' => 'TEST_SVC2',
					'hostgroup_name' => 'TEST_HG',
					'check_command' => 'TEST_COMMAND2',
				),
			),
			'commands' => array(
				array(
					'command_name' => 'TEST_COMMAND1',
					'command_line' => '/bin/echo Weeee',
				),
				array(
					'command_name' => 'TEST_COMMAND2',
					'command_line' => '/bin/echo Wooo',
				),
			),
		);

		$mpm = new management_pack_management_nacoma_Model();
		$this->assertSame($mpm->is_imported($real_pack), 0, 'The management pack should not be imported');
		$res = $mpm->import($real_pack);
		$this->assertTrue(is_numeric($res), "Importing the management pack should return integer, got ".var_export(is_object($res)?$res->validation_error_list:$res, true));
		$this->assertTrue($res > 0, 'Importing the management pack should work, got '.
			var_export(is_object($res)?$res->validation_error_list:$res, true));
		$this->assertSame($mpm->is_imported($real_pack), 1, 'The management pack should be considered imported when asking again');
		$res = $mpm->import($real_pack);
		$this->assertTrue($res > 0, 'Re-importing the management pack should work, got '.
			var_export(is_object($res)?$res->validation_error_list:$res, true));

		$modified_pack = $real_pack;
		$modified_pack['hostgroup']['hostgroup_name'] = 'hostgroup_that_does_not_exist';
		$modified_pack['management_pack']['hostgroup'] = 'hostgroup_that_does_not_exist';
		$modified_pack['management_pack']['management_pack_name'] = 'pack_that_does_not_exist';
		$modified_pack['commands'][] = array('command_name' => 'check-host-alive', 'command_line' => 'this is not the default command line');
		$res = $mpm->import($modified_pack);
		$this->assertSame($res, false, "Importing the management pack with new name/group, but modified command, force-free should complain, got ".
			var_export(is_object($res)?$res->validation_error_list:$res, true));

		$real_pack['services'][]= array(
			'service_description' => 'third svc',
			'hostgroup_name' => 'TEST_HG',
			'check_command' => 'TEST_COMMAND1',
		);
		$this->unimported_check_helper($real_pack, 'with new service');
		$real_pack['commands'][]= array(
			'command_name' => 'TEST_COMMAND3',
			'command_line' => '/bin/echo Waaa',
		);
		$this->unimported_check_helper($real_pack, 'with new command');
		$real_pack['management_pack']['_VAR2'] = 'other value';
		$this->unimported_check_helper($real_pack, 'with new custom var on pack');
		$real_pack['management_pack']['description'] = "Why can't we be friends?";
		$this->unimported_check_helper($real_pack, 'with changed description on pack');

		$real_pack['commands'][0]['command_line'] = 'I changed you';
		$this->trixy_check_helper($real_pack, 'with changed command');
		$real_pack['services'][0]['notes'] = 'Pikachu, I choose you';
		$this->trixy_check_helper($real_pack, 'with changed service');
		$real_pack['hostgroup']['notes'] = 'This note was intentionally left blank.';
		$this->trixy_check_helper($real_pack, 'with changed hostgroup');

		$real_pack['management_pack']['_VAR2'] = 'value';
		unset($real_pack['management_pack']['_VAR']);
		$this->unimported_check_helper($real_pack, 'with changed description on pack');
		$mp = new management_pack_nacoma_Model($real_pack['management_pack']['management_pack_name']);
		$this->assertSame($mp->custom_vars['_VAR2'], 'value', "After renaming custom var, the new name is set");
		$this->assertTrue(!isset($mp->custom_vars['_VAR']), "After renaming custom var, the old name is not set");

		$this->cleanup_helper($real_pack);

		# It's important to bail here for new packs, or 6.2 upgrades would break stuff
		$pack_existing_hg = $real_pack;
		$pack_existing_hg['management_pack']['management_pack_name'] = 'another test pack';
		$pack_existing_hg['hostgroup']['hostgroup_name'] = 'hostgroup_1';
		$res = $mpm->is_imported($pack_existing_hg);
		$this->assertTrue(
			is_array($res),
			"The new management pack with preexisting hostgroup is trixsy, " .
			"so expected exception, got " . print_r($res, true)
		);
		$this->assertSame($mpm->import($pack_existing_hg), false, "The new management pack with preexisting hostgroup can't be imported without force");
	}

	function testMgmtPackImportWithLimitedRights()
	{
		$mpm = new management_pack_management_nacoma_Model();
		$pack = array(
			'management_pack' => array(
				'management_pack_name' => 'CLI_TEST_PACK',
				'description' => "This is not the pack you're looking for",
				'icon' => 'server.png',
				'hostgroup' => 'CLI_TEST_HG',
				'_VAR' => 'value',
			),
			'hostgroup' => array(
				'hostgroup_name' => 'CLI_TEST_HG',
				'notes' => 'what?',
			),
			'services' => array(
				array(
					'service_description' => 'CLI_TEST_SVC1',
					'hostgroup_name' => 'CLI_TEST_HG',
					'check_command' => 'CLI_TEST_COMMAND1',
					'notes' => 'no reason',
				),
			),
			'commands' => array(
				array(
					'command_name' => 'CLI_TEST_COMMAND1',
					'command_line' => '/bin/echo Weeee',
				),
			),
		);
		$ug = new usergroup_nacoma_Model('mgmt_limited_group');
		$ug->obj['access_rights'] = array('management_pack_edit_all', 'management_pack_view_all', 'management_pack_add_delete');
		$this->assertTrue($ug->save_object(), 'New usergroup should be saved, but got: '.var_export($ug->validation_error_list, true));
		set_user('contact_limited', array('mgmt_limited_group'));

		$res = $mpm->import($pack, true);
		$this->assertTrue(is_object($res), "When trying to import a management pack with only management pack rights, I should get an object error back");
		$this->assertNotEmpty($res->validation_error_list, "When trying to import a management pack with only management pack rights, the return should have validation errors");

		$obj = new management_pack_nacoma_Model('CLI_TEST_PACK');
		$this->assertSame(false, $obj->id, "After lacking permissions to import everything, the pack I did have access to import shouldn't be imported");

		$ug->obj['access_rights'] = array('management_pack_edit_all', 'management_pack_view_all', 'management_pack_add_delete', 'command_add_delete');
		$this->assertTrue($ug->save_object(), 'New usergroup should be saved, but got: '.var_export($ug->validation_error_list, true));
		set_user('contact_limited', array('mgmt_limited_group'));

		$res = $mpm->import($pack, true);
		$this->assertTrue(is_object($res), "When trying to import a management pack with only management pack + command rights, I should get an object error back");
		$this->assertNotEmpty($res->validation_error_list, "When trying to import a management pack with only management pack + command rights, the return should have validation errors");

		$obj = new management_pack_nacoma_Model('CLI_TEST_PACK');
		$this->assertSame(false, $obj->id, "After lacking permissions to import everything, the pack I did have access to import shouldn't be imported");
		$obj = new command_nacoma_Model('CLI_TEST_COMMAND1');
		$this->assertSame(false, $obj->id, "After lacking permissions to import everything, the command I did have access to import shouldn't be imported");

		$this->cleanup_helper($pack);
	}

	function testManagementPackCli()
	{
		$pack = array(
			'management_pack' => array(
				'management_pack_name' => 'CLI_TEST_PACK',
				'description' => "This is not the pack you're looking for",
				'icon' => 'server.png',
				'hostgroup' => 'CLI_TEST_HG',
				'_VAR' => 'value',
			),
			'hostgroup' => array(
				'hostgroup_name' => 'CLI_TEST_HG',
				'notes' => 'what?',
			),
			'services' => array(
				array(
					'service_description' => 'CLI_TEST_SVC1',
					'hostgroup_name' => 'CLI_TEST_HG',
					'check_command' => 'CLI_TEST_COMMAND1',
					'notes' => 'no reason',
				),
				array(
					'service_description' => 'CLI_TEST_SVC2',
					'hostgroup_name' => 'CLI_TEST_HG',
					'check_command' => 'CLI_TEST_COMMAND2',
				),
			),
			'commands' => array(
				array(
					'command_name' => 'CLI_TEST_COMMAND1',
					'command_line' => '/bin/echo Weeee',
				),
				array(
					'command_name' => 'CLI_TEST_COMMAND2',
					'command_line' => '/bin/echo Wooo',
				),
			),
		);
		$name = tempnam("/usr/share/monitor/management-packs", "testpack").'.json';
		$this->assertTrue(file_put_contents($name, json_encode(array('management_pack' => $pack, 'vendor'=>'', 'changes'=>array(array('version'=>1,'description'=>'no changes'))))) !== false, "Writing a test pack to file should work");
		exec("php ./mgmtpackimport.php --import --name ".basename($name, '.json'), $out, $return);
		$this->assertSame($return, 0, "Exit code from importing mgmtpack should be 0");
		$this->assertEmpty(trim(implode("\n", $out)), "Expected import to be OK (empty output), got ".var_export($out, true));
		$this->cleanup_helper($pack);

		$out=false;
		$pack['hostgroup']['hostgroup_name'] = 'hostgroup_1';
		$hg = new hostgroup_nacoma_Model($pack['hostgroup']['hostgroup_name']);
		$this->assertTrue($hg->id !== false, "The hostgroup used for testing for preexisting hostgroup should exist");
		$this->assertTrue(file_put_contents($name, json_encode(array('management_pack' => $pack, 'vendor'=>'', 'changes'=>array(array('version'=>1,'description'=>'no changes'))))) !== false, "Writing a test pack to file should work");
		exec("php ./mgmtpackimport.php --import --name ".basename($name, '.json'), $out, $return);
		$this->assertSame($return, 1, "Exit code from importing mgmtpack with already-existing hostgroup should be 1");
		$this->assertEquals($out[0], "Cannot import management pack 'CLI_TEST_PACK', because it has been modified. You can try to force the import, but that might cause checks to change or fail.", "If the hostgroup already exist, we should not import without warning");
		unlink($name);
	}

	function testInUseManagementPack()
	{
		$mpm = new management_pack_management_nacoma_Model();
		$pack = array(
			'management_pack' => array(
				'management_pack_name' => 'TEST_PACK',
				'description' => "This is not the pack you're looking for",
				'icon' => 'server.png',
				'hostgroup' => 'TEST_HG',
				'_VAR' => 'value',
			),
			'hostgroup' => array(
				'hostgroup_name' => 'TEST_HG',
				'notes' => 'what?',
			),
			'services' => array(
				array(
					'service_description' => 'TEST_SVC1',
					'hostgroup_name' => 'TEST_HG',
					'check_command' => 'TEST_COMMAND1',
					'notes' => 'no reason',
				),
			),
			'commands' => array(
				array(
					'command_name' => 'TEST_COMMAND1',
					'command_line' => '/bin/echo Weeee',
				),
			),
		);
		$res = $mpm->is_imported($pack);
		$this->assertSame(0, $res, "Expected management pack not to be imported, got $res");
		$res = $mpm->import($pack);
		$this->assertTrue(is_numeric($res), "Importing the management pack force-free should not complain: ". var_export(is_object($res)?$res->validation_error_list:$res, true));
		$this->assertTrue($res > 0, "Importing the management pack force-free should not complain");
		$hst = new host_nacoma_Model();
		$hst->get_default_object();
		$hst->obj['host_name'] = 'host that uses test pack';
		$hst->normalize(array('hostgroups' => array('TEST_HG')));
		$this->assertTrue($hst->save_object(), 'Saving should work fine, got '.var_export($hst->validation_error_list, true));
		$pack['commands'][] = array(
			'command_name' => 'TEST_COMMAND2',
			'command_line' => '/bin/echo Whaat?',
		);
		$res = $mpm->is_imported($pack);
		$this->assertSame(0, $res, "Expected management pack not to be imported, got $res");
		$res = $mpm->import($pack);
		$this->assertTrue($res > 0, "Importing the management pack force-free should not complain");
		$res = $mpm->is_imported($pack);
		$hst = new host_nacoma_Model('host that uses test pack');
		$this->assertTrue($hst->id > 0, 'Host should be found after upgrade');
		$hst_contents = $hst->denormalize();
		$this->assertEquals($hst_contents['hostgroups'][0], 'TEST_HG', 'Host should keep hostgroup association after upgrade');

		$this->cleanup_helper($pack);
	}
}
