<?php
require_once (__DIR__ . '/../../include/config.inc.php');

/**
 * Test higher level functionality in object_nacoma_Model
 * Lower level tests are in oatest.php
 */
class ObjlockTest extends PHPUnit_Framework_TestCase {
	protected $backupGlobalsBlacklist = array (
		'master_ary',
		'Supported_Object_Types'
	);

	public function setUp() {
		op5objstore::instance()->mock_clear();
	}

	public function tearDown() {
		op5objstore::instance()->mock_clear();
		$sql = 'DELETE FROM objlocks';
		$res = nacoma_db::prepare_statement($sql);
		$res->execute();
	}

	/**
	 * test that locks block later locks
	 */
	public function test_lock_block_unlock_unblock() {
		set_user( 'objlocktest_user' );

		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );

		nacoma_objlock::free_lock( 'test_lock_a', 0 );

		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
	}

	/**
	 * Test that multiple object types can be locked independently
	 */
	public function test_lock_unlock_multiple_types() {
		set_user( 'objlocktest_user' );

		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_b', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_b', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_b', 0 ) );

		nacoma_objlock::free_lock( 'test_lock_b', 0 );

		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_b', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_b', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_b', 0 ) );

		nacoma_objlock::free_lock( 'test_lock_a', 0 );
		nacoma_objlock::free_lock( 'test_lock_b', 0 );

		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_b', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_b', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_b', 0 ) );
	}

	/**
	 * Test that multiple object id:s can be locked independently
	 */
	public function test_lock_unlock_multiple_ids() {
		set_user( 'objlocktest_user' );

		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 1 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 1 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 1 ) );

		nacoma_objlock::free_lock( 'test_lock_a', 1 );

		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 1 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 1 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 1 ) );

		nacoma_objlock::free_lock( 'test_lock_a', 0 );
		nacoma_objlock::free_lock( 'test_lock_a', 1 );

		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 1 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 1 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 1 ) );
	}

	/**
	 * Test if locks can timeout
	 */
	public function test_timeout() {
		set_user( 'objlocktest_user' );

		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 0, 2 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0, 2 ) );
		sleep( 3 );
		$this->assertTrue( nacoma_objlock::extend_lock( 'test_lock_a', 0, 2 ) );
		$this->assertFalse( nacoma_objlock::extend_lock( 'test_lock_a', 0, 2 ) );
	}

	/**
	 * Test that get lock owner returns owner name.
	 */
	public function test_lock_ownership() {
		set_user('objlocktest_user_a');
		$this->assertTrue(nacoma_objlock::extend_lock('test_lock_a', 0));
		$this->assertSame(
			'objlocktest_user_a',
			nacoma_objlock::get_lock_owner('test_lock_a', 0)
		);

		set_user('objlocktest_user_b');
		$this->assertSame(
			'objlocktest_user_a',
			nacoma_objlock::get_lock_owner('test_lock_a', 0)
		);

		set_user('objlocktest_user_a');
		$this->assertSame(
			'objlocktest_user_a',
			nacoma_objlock::get_lock_owner('test_lock_a', 0
		));
	}

	/**
	 * Test that locks work as predicted when using seperate processes and same user.
	 */
	public function test_lock_flow() {
		set_user('sleeper_user');

		// Assert that no one has a lock.
		$this->assertSame(false, nacoma_objlock::get_lock_owner('all', 0));

		// Start a background process that locks.
		exec('/usr/bin/php ' . __DIR__ . '/../lock_and_sleep_one.php > /dev/null &');

		// Wait for lock to be set.
		$now = microtime(true);
		while (!$user = nacoma_objlock::get_lock_owner('all', 0)) {
			if (microtime(true) - $now > 5) {
				// Time out after 5 seconds. Fail test.
				$this->fail('get_lock_owner check timed out. Couldn\'t get lock.');
			}
		}

		$this->assertSame('sleeper_user', $user);

		// Allow the lock to be released.
		$now = microtime(true);
		while (nacoma_objlock::get_lock_owner('all', 0)) {
			if (microtime(true) - $now > 5) {
				// Time out after 5 seconds. Fail test.
				$this->fail('get_lock_owner check timed out. Couldn\'t get lock.');
			}
		}
	}

	/**
	 * Test that the "all"-lock is returned for other lock owners
	 */
	public function test_all() {
		set_user('objlocktest_user_a');
		$this->assertTrue(nacoma_objlock::extend_lock('all', 0));
		$this->assertSame(
			'objlocktest_user_a',
			nacoma_objlock::get_lock_owner('x', false)
		);
		$this->assertSame(
			'objlocktest_user_a',
			nacoma_objlock::get_lock_owner('test_lock_a', 0)
		);
		$this->assertSame(
			'objlocktest_user_a',
			nacoma_objlock::get_lock_owner('test_lock_b', 0)
		);
		set_user('objlocktest_user_b');
		$this->assertSame(
			'objlocktest_user_a',
			nacoma_objlock::get_lock_owner('x', false)
		);
		$this->assertSame(
			'objlocktest_user_a',
			nacoma_objlock::get_lock_owner('test_lock_a', 0)
		);
		$this->assertSame(
			'objlocktest_user_a',
			nacoma_objlock::get_lock_owner('test_lock_b', 0)
		);

		set_user('objlocktest_user_a');
		nacoma_objlock::free_lock('all', 0);
		set_user('objlocktest_user_b');
		$this->assertSame(
			false,
			nacoma_objlock::get_lock_owner('x', false)
		);
		$this->assertSame(
			false,
			nacoma_objlock::get_lock_owner('test_lock_a', 0)
		);
		$this->assertSame(
			false,
			nacoma_objlock::get_lock_owner('test_lock_b', 0)
		);
	}
}
